import copy
from datetime import datetime


def prepare_order(
    d,
    price=None,
    lots=None,
    demo=True,
    order_note=None,
    order_status=None,
):
    d2 = copy.deepcopy(d)
    trade_command_phrase = "pass"
    order_id = d2.get("order_id", None)

    if demo or order_id is None or order_id == "DEMO":
        d2["order_id"] = "DEMO" if demo else None
        d2["price"] = price
        d2["lots"] = lots
        d2["order_status"] = "active" if order_status is None else order_status
        if demo:
            d2["ts"] = datetime.now().timestamp()
        else:
            trade_command_phrase = "place_order"
    else:
        update_order = False
        if price != d["price"] and price is not None:
            d2["price"] = price
            update_order = True
        if lots != d["lots"] and lots is not None:
            d2["lots"] = lots
            update_order = True
        if order_status != d["order_status"] and order_status is not None:
            d2["order_status"] = order_status
            update_order = True
        if update_order:
            trade_command_phrase = "update_order"

    if order_note is not None:
        d["order_note"] = order_note

    return d2, trade_command_phrase


def send_trade_order(client, d, command_phrase):
    # if "lots" in d.keys():
    #     d["lots"] = abs(d["lots"])

    if command_phrase == "place_order":
        res = client.place_order(d)

        if res["status"] == "success":
            d["order_id"] = res["order_id"]
            d["ts"] = datetime.now().timestamp()
        else:
            raise Exception("Order could not be placed")

    elif command_phrase == "update_order":
        res = client.update_order(
            d=d,
        )
        if res["status"] == "success":
            # d["order_id"] = res[0]["response"]
            d["ts"] = datetime.now().timestamp()
        elif res["detail"] == "partial_match_occured":
            d["order_status"] = "cancelled"
            res = client.update_order(d=d)
            if res["status"] == "success":
                d["order_id"] = None
        elif res["detail"] == "order_cannot_be_updated":
            d["order_id"] = None
        else:
            raise Exception("Order could not be placed")
    elif command_phrase == "cancel_order":
        d["order_status"] = "cancelled"
        res = client.update_order(d=d)
        if res["status"] == "success":
            d["order_id"] = None

    else:
        raise Exception("Invalid command phrase")

    return res, d



import requests
import os
import sseclient
from datetime import datetime


class RKClient:
    def __init__(self, creds, server="https://idm.robokami.com", **kwargs) -> None:
        self.server = server
        self.creds = creds
        if kwargs.get("initiate_login", True):
            self.authorize()

        if kwargs.get("initiate_stream", False) and self.session_token is not None:
            self.stream()

    def authorize(self):
        res = requests.get(
            os.path.join(self.server, "login"),
            json={"credentials": self.creds},
            timeout=15,
        )

        if res.status_code == 200:
            self.session_token = res.json()["session_token"]
            self.iat = datetime.now().timestamp()
        else:
            print("Login failed")
            self.session_token = None

    def renew_session(self):
        self.authorize()

    def place_order(self, d):
        d["order_status"] = d.get("order_status", "active")
        d["order_note"] = d.get("order_note", "RK-TRADER")
        return self.trade_command("place_order", d)

    def update_order(self, d):
        if "order_id" not in d.keys():
            return {"status": "error", "message": "order_id is required"}
        d["contract_type"] = "hourly" if d["c"].startswith("PH") else "block"
        d["order_note"] = d.get("order_note", "RK-TRADER")
        return self.trade_command("update_order", d)

    def trade_command(self, command, d):
        res = requests.post(
            os.path.join(self.server, "trade", command),
            headers={"Authorization": self.session_token},
            json=d,
        )

        if res.status_code == 200:
            return res.json()
        else:
            print("Failed response code: " + str(res.status_code))
            print(res.json())
            return res.json()

    def stream(self):
        response = requests.get(
            os.path.join(self.server, "stream"),
            headers={"Authorization": self.session_token},
            stream=True,
        )

        self.stream_client = sseclient.SSEClient(response)



from setuptools import setup, find_packages
from pkg_resources import parse_requirements
from robokami import robokami_version

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="robokami-py",
    version=robokami_version,
    author="Robokami",
    description="Robokami Client Python SDK",
    packages=find_packages(),
    install_requires=["sseclient_py==1.7.2"],
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="https://github.com/Tideseed/robokami-py",
    license="Apache License 2.0",
    # classifiers like program is suitable for python3, just leave as it is.
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    # Install requires get from requirements.txt
    # install_requires=[
    #     str(r.req) for r in parse_requirements("requirements.txt", session=False)
    # ],
)


## To create a requirements.txt file from the project: pipreqs . --force
## To install from requirements.txt: pip install -r requirements.txt OR uncomment the install_requires line above
